unit Main;

{-------------------------------------------------------------------------------

BHOCaptor - Controls browser helper objects
Copyright (C) 2000 Adam Stiles.

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

-------------------------------------------------------------------------------}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, StdCtrls, Registry, ComCtrls, ExtCtrls, ShellApi;

type
  TMainForm = class(TForm)
    BHOList: TListView;
    TopPanel: TPanel;
    FollowingLabel: TLabel;
    BottomPanel: TPanel;
    DeactivateButton: TButton;
    DeactLabel1: TLabel;
    DeactLabel2: TLabel;
    DoubleClickLabel: TLabel;
    AboutButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure BHOListChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure DeactivateButtonClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BHOListDblClick(Sender: TObject);
    procedure AboutButtonClick(Sender: TObject);
  private
    { Private declarations }
    procedure LoadBHOs;
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

const
  VERNUM = '0.5';

implementation

{$R *.DFM}
//------------------------------------------------------------------------------
procedure TMainForm.LoadBHOs;
const
  BHO_KEY='SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Browser Helper Objects';
var
  Reg: TRegistry;
  Keys: TStringList;
  Key: String;
  I: Integer;
  Item: TListItem;
  Desc, FileName: String;
begin
  BHOList.Items.Clear;
  Keys:=TStringList.Create;
  try
    Reg:=TRegistry.Create;
    try
      Reg.RootKey:=HKEY_LOCAL_MACHINE;
      if Reg.OpenKey(BHO_KEY, false) then Reg.GetKeyNames(keys);
    finally
      Reg.Free;
    end;

    Reg:=TRegistry.Create;
    try
      reg.RootKey:=HKEY_CLASSES_ROOT;
      for I :=0 to Keys.Count-1 do
      begin
        Desc := '';
        FileName := '';
        Key := Keys[i];
        if Reg.OpenKey('CLSID\'+Key, False) then
        begin
          Desc := Reg.ReadString('');
          Reg.CloseKey;
          // should probably report an error or something here
          if Reg.OpenKey('CLSID\'+Key+'\InprocServer32', False) then
          begin
            FileName := Reg.ReadString('');
            Reg.CloseKey;
          end;
        end
        else
          Desc := Key;
        Item := BHOList.Items.Add;
        Item.Caption := Desc;
        Item.SubItems.Add(FileName);
        Item.SubItems.Add(Key);
      end;
    finally
      Reg.Free;
    end;
  finally
    Keys.Free;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.FormCreate(Sender: TObject);
var
  NCMetrics: TNonClientMetrics;
begin
  Caption := 'BHOCaptor ' + VERNUM;

  // load default font
  NCMetrics.cbSize := sizeof(NCMetrics);
  if (SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NCMetrics, 0)) then
    Font.Handle := CreateFontIndirect(NCMetrics.lfMessageFont);

  LoadBHOs;
end;
//------------------------------------------------------------------------------
procedure TMainForm.BHOListChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  // enable the deactivate button if there are items selected
  if (Change=ctState) then
    DeactivateButton.Enabled := (BHOList.SelCount <> 0);
end;
//------------------------------------------------------------------------------
procedure TMainForm.DeactivateButtonClick(Sender: TObject);
var
  Item: TListItem;
  Key: string;
  Reg: TRegistry;
begin
  if (MessageDlg('Are you sure you want to deactivate this Browser Helper Object?',
    mtConfirmation, [mbYes, mbNo],0) = mrYes) then
  begin
    Item:=BHOList.Selected;
    if Assigned(Item) then
    begin
      if (Item.SubItems.count > 1) then
      begin
        Key := Item.Subitems[1];
        Reg := TRegistry.Create;
        try
          Reg.RootKey := HKEY_LOCAL_MACHINE;
          // should probably error here if we didn't delete the key
          if Reg.DeleteKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Browser Helper Objects\'+key) then
            LoadBHOs;
        finally
          Reg.Free;
        end;
      end;
    end;
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.FormDestroy(Sender: TObject);
begin
  BHOList.Items.Clear;
end;
//------------------------------------------------------------------------------
procedure TMainForm.BHOListDblClick(Sender: TObject);
var
  Item: TListItem;
  Info: TShellExecuteInfo;
  FileName: String;
begin
  // show the selected file's properties dialog
  Item := BHOList.Selected;
  if Assigned(Item) then
  begin
    FileName := Item.SubItems[0];
    with Info do
    begin
      cbSize := SizeOf(Info);
      fMask := SEE_MASK_NOCLOSEPROCESS or
        SEE_MASK_INVOKEIDLIST or
        SEE_MASK_FLAG_NO_UI;
      wnd := Handle;
      lpVerb := 'properties';
      lpFile := pChar(FileName);
      lpParameters := nil;
      lpDirectory := nil;
      nShow := 0;
      hInstApp := 0;
      lpIDList := nil;
    end;
    ShellExecuteEx(@Info);
  end;
end;
//------------------------------------------------------------------------------
procedure TMainForm.AboutButtonClick(Sender: TObject);
begin
  MessageDlg('BHOCaptor' + #13#10 + 'Copyright (c) 2000 Adam Stiles. All '
    + 'Rights Reserved.' + #13#10#13#10
    + 'See http://www.xcaptor.org for updates.',
    mtInformation, [mbOK], 0);
end;
//------------------------------------------------------------------------------
end.
